/*******************************************************************************
 * Copyright (c) 2008 The University of York.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Dimitrios Kolovos - initial API and implementation
 *     H�ctor Iturria S�nchez - Modifications in methods inputs and code
 ******************************************************************************/
package es.cv.gvcase.linkers.test.evl;

import java.io.*;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Platform;
import org.eclipse.epsilon.commons.parse.problem.ParseProblem;
import org.eclipse.epsilon.commons.util.OperatingSystem;
import org.eclipse.epsilon.commons.util.StringProperties;
import org.eclipse.epsilon.emc.emf.EmfModel;
import org.eclipse.epsilon.eol.IEolExecutableModule;
import org.eclipse.epsilon.eol.exceptions.models.EolModelLoadingException;
import org.eclipse.epsilon.eol.models.IModel;

public abstract class EpsilonStandaloneFile {
	
	public String namePlugin = null;
	
	protected IEolExecutableModule module;
	
	public abstract IEolExecutableModule createModule();
	
	public abstract List<IModel> getModels(String modelFile, String metamodelFile, String metamodelUri, String dirModels) throws Exception;
	
	public abstract boolean postProcess();
	
	public abstract boolean postProcess(String fileName, PrintWriter output);
	
	public boolean execute(String modelFile, String metamodelFile, String metamodelUri, String dirModels, String dirValidations, String validationFile, PrintWriter output) throws Exception {
		
		module = createModule();
		
		String validationPath = FileLocator.toFileURL(Platform.getBundle(namePlugin).getResource("validations/" +validationFile)).getPath();
		
		//We can pass the name of the file instead calling the method getSource()
		module.parse(new File(validationPath));
		
		if (module.getParseProblems().size() > 0) {
			System.err.println("Parse errors occured...");
			for (ParseProblem problem : module.getParseProblems()) {
				System.err.println(problem.toString());
			}
			System.exit(-1);
		}
		
		for (IModel model : getModels(modelFile, metamodelFile, metamodelUri, dirModels)) {
			module.getContext().getModelRepository().addModel(model);
		}
		
		module.execute();
		
		//hasErrors contains if errors exist when validating
		boolean hasErrors = postProcess(validationFile, output);
		
		module.getContext().getModelRepository().dispose();
		
		//returns a value in order to have a feedback in JUnit
		return hasErrors;
	}
	
	protected EmfModel createEmfModel(String name, String model, String metamodel, boolean readOnLoad, boolean storeOnDisposal) throws EolModelLoadingException, URISyntaxException {
		
		String dirModels = "";
		String dirMetamodels = "";
		
		EmfModel emfModel = new EmfModel();
		StringProperties properties = new StringProperties();
		properties.put(EmfModel.PROPERTY_NAME, name);
		properties.put(EmfModel.PROPERTY_METAMODEL_FILE, new File(dirMetamodels + metamodel).getAbsolutePath());
		properties.put(EmfModel.PROPERTY_MODEL_FILE, new File(dirModels + model).getAbsolutePath());
		properties.put(EmfModel.PROPERTY_IS_METAMODEL_FILE_BASED, "true");
		properties.put(EmfModel.PROPERTY_READONLOAD, readOnLoad + "");
		properties.put(EmfModel.PROPERTY_STOREONDISPOSAL, storeOnDisposal + "");
		emfModel.load(properties, null);
		return emfModel;
	}

	protected EmfModel createEmfModelByURI(String name, String model, String metamodel, String dirModels, boolean readOnLoad, boolean storeOnDisposal) throws EolModelLoadingException, URISyntaxException {
		
		String wsRoot = null;
		String propertyModelFile = null;
		
		//In order to run the testing in Linux the path of each model should be expressed in two different ways:
		//Windows: absolute path
		//Linux: workspace relative path
		if(OperatingSystem.isWindows()){
			wsRoot = Platform.getLocation().toString();
			propertyModelFile = new File(wsRoot + dirModels + model).getAbsolutePath();
		}
		else if(OperatingSystem.isUnix()){
			//propertyModelFile = new File(dirModels + model).getAbsolutePath();
			//wsRoot = Platform.getLocation().toString();
			wsRoot = System.getProperty("user.home");
			String wkspace = wsRoot + "/tempWorkspace/" + namePlugin;
			String allPath = "file:/" + wkspace + dirModels + model;
			propertyModelFile = new File(allPath).toString();
		}
		
		EmfModel emfModel = new EmfModel();
		StringProperties properties = new StringProperties();
		properties.put(EmfModel.PROPERTY_NAME, name);
		properties.put(EmfModel.PROPERTY_METAMODEL_URI, metamodel);
		properties.put(EmfModel.PROPERTY_MODEL_FILE, propertyModelFile);
		properties.put(EmfModel.PROPERTY_IS_METAMODEL_FILE_BASED, "false");
		properties.put(EmfModel.PROPERTY_READONLOAD, readOnLoad + "");
		properties.put(EmfModel.PROPERTY_STOREONDISPOSAL, storeOnDisposal + "");
		emfModel.load(properties, null);
		return emfModel;
	}
	
	protected File getFile(String fileName) throws URISyntaxException {
		
		URI binUri = EpsilonStandaloneFile.class.getResource(fileName).toURI();
		URI uri = null;
		
		if (binUri.toString().indexOf("bin") > -1) {
			uri = new URI(binUri.toString().replaceAll("bin", "src"));
		}
		else {
			uri = binUri;
		}
		System.out.println(uri);
		return new File(uri);
	}
	
}
