/***************************************************************************
* Copyright (c) 2007 Conselleria de Infraestructuras y Transporte,
* Generalitat de la Comunitat Valenciana . All rights reserved. This program
* and the accompanying materials are made available under the terms of the
* Eclipse Public License v1.0 which accompanies this distribution, and is
* available at http://www.eclipse.org/legal/epl-v10.html
*
* Contributors: H�ctor Iturria S�nchez - Initial API and implementation
*				Jesús Badenas Martínez
******************************************************************************/

package es.cv.gvcase.linkers.test.junit;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.io.*;

import org.eclipse.core.runtime.Platform;

import es.cv.gvcase.linkers.test.evl.*;
import es.cv.gvcase.trmanager.TransformedResource;
import es.cv.gvcase.trmanager.registry.TransformationDesc;
import es.cv.gvcase.trmanager.registry.TransformationsRegistry;
import es.cv.gvcase.mdt.common.util.CopyUtils;

public class TransformAndValidate {
	
	String transformationID;
	String dirModels;
	String dirValidations;
	String metamodelFile;
	String metamodelUri;
	String transformationInputPath;
	String transformationOutputPath;
	String dirConfigurationModels;
	EvlStandaloneMetamodelByUriFile evl;
	String extension;
	String nameOfPlugin;
	public int nErrors;
	
	public TransformAndValidate(String trID, String dModels, String dValidations, String mmFile, String mmUri, String tIP, String tOP, String ext){
		this.transformationID = trID;
		this.dirModels = dModels;
		this.dirValidations = dValidations;
		this.metamodelFile = mmFile;
		this.metamodelUri = mmUri;
		this.transformationInputPath = tIP;
		this.transformationOutputPath = tOP;
		this.evl = new EvlStandaloneMetamodelByUriFile();
		this.extension = ext;
	}	
	
	public TransformAndValidate(String trID, String dModels, String dValidations, String mmFile, String mmUri, String tIP, String tOP, String dCM, String ext, String namePlug){
		this.transformationID = trID;
		this.dirModels = dModels;
		this.dirValidations = dValidations;
		this.metamodelFile = mmFile;
		this.metamodelUri = mmUri;
		this.transformationInputPath = tIP;
		this.transformationOutputPath = tOP;
		this.dirConfigurationModels = dCM;
		this.evl = new EvlStandaloneMetamodelByUriFile();
		this.extension = ext; 
		this.nameOfPlugin = namePlug;
		this.evl.namePlugin = namePlug;
		this.nErrors = 0;
	}
	
	public boolean execute(String fileName, PrintWriter ol){

		TransformationDesc tr = TransformationsRegistry.getTransformation(transformationID);
		ArrayList<String> messagesList = new ArrayList<String>();
		HashMap<String, TransformedResource> inputParams = new HashMap<String, TransformedResource>();
		HashMap<String, TransformedResource> outputParams = new HashMap<String, TransformedResource>();
		
		inputParams.put("projectInputParam", new TransformedResource("projectInputParam", transformationInputPath + fileName + ".model"));
		outputParams.put("fileOutputParam", new TransformedResource("fileOutputParam", transformationOutputPath + fileName + "." + extension));
		
		boolean errors = true;
		System.out.println("\nTransforming: " + fileName);
		boolean execOK = TransformationsRegistry.executeTransformation(tr.getId(), inputParams, outputParams, messagesList);
		try {
			if (execOK == true){
				System.out.println("Validating: " + fileName);
					errors = evl.execute(fileName + "." + extension, metamodelFile, metamodelUri, dirModels, dirValidations, fileName + ".evl", ol);
			}
			else {
				OutputLog.addToLog("\nIn validation " + fileName, ol);
				OutputLog.addToLog("Can't transform " + fileName + " succesfully.", ol);
				System.out.println("Can't transform " + fileName + " succesfully.");
				OutputLog.addToLog("Reasons:", ol);
				System.out.println("Reasons:\n");
				
				Iterator<String> iter = messagesList.iterator();
				
				while(iter.hasNext()){
					String message = iter.next();
					System.out.println(message);
					OutputLog.addToLog(message, ol);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			nErrors++;
			OutputLog.addToLog("\nIn validation " + fileName, ol);
			OutputLog.addToLog(e.getMessage(), ol);
		}
    	return errors;
	}
	
	public boolean execute(String fileName, String configModel, PrintWriter ol){

		TransformationDesc tr = TransformationsRegistry.getTransformation(transformationID);
		ArrayList<String> messagesList = new ArrayList<String>();
		HashMap<String, TransformedResource> inputParams = new HashMap<String, TransformedResource>();
		HashMap<String, TransformedResource> outputParams = new HashMap<String, TransformedResource>();
		TransformedResource configuration = new TransformedResource("CONFIGURATION", dirConfigurationModels + fileName + "_" + configModel + ".transformationconfiguration");
		
		inputParams.put("projectInputParam", new TransformedResource("projectInputParam", transformationInputPath + fileName + ".model"));
		outputParams.put("fileOutputParam", new TransformedResource("fileOutputParam", transformationOutputPath + fileName + "_" + configModel + "."  + extension));
		
		boolean errors = true;
		System.out.println("\nTransforming: " + fileName + "_" + configModel);
		boolean execOK = TransformationsRegistry.runTransformation(tr.getId(), inputParams, outputParams, messagesList, null, configuration);
		try {
			if (execOK == true){
					System.out.println("Validating: " + fileName + "_" + configModel);
					errors = evl.execute(fileName + "_" + configModel + "." + extension, metamodelFile, metamodelUri, dirModels, dirValidations, fileName + "_" + configModel + ".evl", ol);
			}
			else {
				OutputLog.addToLog("\nIn validation " + fileName, ol);
				OutputLog.addToLog("Can't transform " + fileName + " succesfully.", ol);
				System.out.println("Can't transform " + fileName + " succesfully.");
				OutputLog.addToLog("Reasons:", ol);
				System.out.println("Reasons:\n");
				
				Iterator<String> iter = messagesList.iterator();
				
				while(iter.hasNext()){
					String message = iter.next();
					System.out.println(message);
					OutputLog.addToLog(message, ol);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			nErrors++;
			OutputLog.addToLog("\nIn validation " + fileName, ol);
			OutputLog.addToLog(e.getMessage(), ol);
		}
    	return errors;
	
	}
	
	public boolean execute(String fileName, String middleModel, String configModel, PrintWriter ol){

		TransformationDesc tr = TransformationsRegistry.getTransformation(transformationID);
		ArrayList<String> messagesList = new ArrayList<String>();
		HashMap<String, TransformedResource> inputParams = new HashMap<String, TransformedResource>();
		HashMap<String, TransformedResource> outputParams = new HashMap<String, TransformedResource>();
		TransformedResource configuration = null;
		boolean execOk = false;
		
		if(configModel != null)	{
			configuration = new TransformedResource("CONFIGURATION", dirConfigurationModels + fileName + "_" + configModel + ".transformationconfiguration");
		}
		
		String wsRoot = null;
		File srcModelFile, dstModelFile = null;
		
		//StringBuilder inputModelsDir = new StringBuilder(transformationInputPath);
		//inputModelsDir.delete(inputModelsDir.indexOf("platform:/plugin"), "platform:/plugin".length());
		
		// The middleModel should be copied to a file with the same name of the current input model
		//wsRoot = Platform.getLocation().toString();
		wsRoot = System.getProperty("user.home");
		
		//srcModelFile = new File(wsRoot + inputModelsDir.toString() + middleModel + ".model");
		
		String pluginPath = Platform.getBundle(nameOfPlugin).getLocation();
		StringBuilder plugPath = new StringBuilder(pluginPath);
		plugPath.delete(plugPath.indexOf("reference:file:"), "reference:file:".length());
		
		srcModelFile = new File(plugPath.toString() + "models/" + middleModel + ".model");
		
		if(configModel != null) {
			dstModelFile = new File(wsRoot + "/tempWorkspace/" + nameOfPlugin + dirModels + fileName + "_" + configModel + "." + extension);
		}
		else {
			dstModelFile = new File(wsRoot + "/tempWorkspace/" + nameOfPlugin + dirModels + fileName + "." + extension);
		}
		
		CopyUtils.copyFile(srcModelFile, dstModelFile);
		
		inputParams.put("projectInputParam", new TransformedResource("projectInputParam", transformationInputPath + fileName + ".model"));
		
		if(configModel != null) {
			inputParams.put("middleModel", new TransformedResource("middleModel", transformationOutputPath + fileName + "_" + configModel + "." + extension));
			outputParams.put("fileOutputParam", new TransformedResource("fileOutputParam", transformationOutputPath + fileName + "_" + configModel + "."  + extension));
		}
		else {
			inputParams.put("middleModel", new TransformedResource("middleModel", transformationOutputPath + fileName + "." + extension));
			outputParams.put("fileOutputParam", new TransformedResource("fileOutputParam", transformationOutputPath + fileName + "."  + extension));
		}
		
		boolean errors = true;
		
		if(configModel != null) {
			System.out.println("\nTransforming: " + fileName + "_" + configModel);
		}
		else {
			System.out.println("\nTransforming: " + fileName);
		}
			
		if(configuration != null) {
			execOk = TransformationsRegistry.runTransformation(tr.getId(), inputParams, outputParams, messagesList, null, configuration);
		}
		else {
			execOk = TransformationsRegistry.executeTransformation(tr.getId(), inputParams, outputParams, messagesList);
		}
		
		try {
			if (execOk == true){
				if(configModel != null) {
					System.out.println("Validating: " + fileName + "_" + configModel);
					errors = evl.execute(fileName + "_" + configModel + "." + extension, metamodelFile, metamodelUri, dirModels, dirValidations, fileName + "_" + configModel + ".evl", ol);
				}
				else {
					System.out.println("Validating: " + fileName);
					errors = evl.execute(fileName + "." + extension, metamodelFile, metamodelUri, dirModels, dirValidations, fileName + ".evl", ol);
				}
			}
			else {
				OutputLog.addToLog("\nIn validation " + fileName, ol);
				OutputLog.addToLog("Can't transform " + fileName + " succesfully.", ol);
				System.out.println("Can't transform " + fileName + " succesfully.");
				OutputLog.addToLog("Reasons:", ol);
				System.out.println("Reasons:\n");
				
				Iterator<String> iter = messagesList.iterator();
				
				while(iter.hasNext()){
					String message = iter.next();
					System.out.println(message);
					OutputLog.addToLog(message, ol);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			nErrors++;
			OutputLog.addToLog("\nIn validation " + fileName, ol);
			OutputLog.addToLog(e.getMessage(), ol);
		}
    	return errors;
	
	}
}
