/***************************************************************************
 * Copyright (c) 2007 Conselleria de Infraestructuras y Transporte,
 * Generalitat de la Comunitat Valenciana . All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors: Jesús Badenas Martínez - Initial API and implementation
 *
 ******************************************************************************/

package es.cv.gvcase.mdt.sketcher.sketcher2uim.test;

import static org.junit.Assert.*;

import java.io.*;

import org.eclipse.emf.ecore.EPackage;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import es.cv.gvcase.linkers.test.junit.*;
import es.cv.gvcase.mdt.uim.UimPackage;


public class TransformationAndValidation {

	static String metamodelFile, metamodelUri, dirValidationInputModels,
			dirValidations, validationFile, transformationId,
			transformationInputPath, transformationOutputPath,
			dirConfigurationModels, namePlugin;
	
	static TransformAndValidate tav;
	
	static PrintWriter ol = OutputLog.startLog("Sketcher2UIM-ErrorLog.txt");
	
	static int nTests, nHits, nFailures;
	static long start, elapsed;
	static double timeTests;

	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		nTests = nHits = nFailures = 0;
		
		start = System.currentTimeMillis();
		
		metamodelFile = "uim_1.3.0.ecore";
		metamodelUri = "http:///es.cv.gvcase.mdt.uim/1.3.0";
		dirValidationInputModels = "/outputmodels/";
		transformationInputPath = "platform:/plugin/es.cv.gvcase.mdt.sketcher.sketcher2uim.test/models/";
		namePlugin = "es.cv.gvcase.mdt.sketcher.sketcher2uim.test";
		
		//String wkspace = Platform.getLocation().toString();
		String home = System.getProperty("user.home");
		String wkspace = home + "/tempWorkspace/" + namePlugin;
		
		transformationOutputPath = "file:/" + wkspace + "/outputmodels/";
		dirConfigurationModels = "file:/" + wkspace + "/confmodels/";
		dirValidations = "platform:/plugin/es.cv.gvcase.mdt.sketcher.sketcher2uim.test/validations/";
		transformationId = "es.cv.gvcase.mdt.sketcher.sketcher2uim.Sketcher2UIM";
		
		EPackage.Registry.INSTANCE.put(metamodelFile, UimPackage.eINSTANCE);
		
		tav = new TransformAndValidate(transformationId,
				dirValidationInputModels, dirValidations, metamodelFile,
				metamodelUri, transformationInputPath,
				transformationOutputPath, dirConfigurationModels, "uim", namePlugin);
	}

	/*@Before
	public void setUp() throws Exception {
	}*/

	@Test
	public void TFR_MAP_001_V00_001() {
		boolean error;
		error = tav.execute("TFR-MAP-001-V00-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_001_V00_001
	
	@Test
	public void TFR_MAP_001_V01_001() {
		boolean error;
		error = tav.execute("TFR-MAP-001-V01-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_001_V01_001
	
	@Test
	public void TFR_MAP_001_V02_001() {
		boolean error;
		error = tav.execute("TFR-MAP-001-V02-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_001_V02_001
	
	@Test
	public void TFR_MAP_001_V03_001() {
		boolean error;
		error = tav.execute("TFR-MAP-001-V03-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_001_V03_001
	
	@Test
	public void TFR_MAP_001_V04_001() {
		boolean error;
		error = tav.execute("TFR-MAP-001-V04-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_001_V04_001
	
	@Test
	public void TFR_MAP_002_V00_001() {
		boolean error;
		error = tav.execute("TFR-MAP-002-V00-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_002_V00_001
	
	@Test
	public void TFR_MAP_002_V01_001() {
		boolean error;
		error = tav.execute("TFR-MAP-002-V01-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_002_V01_001
	
	@Test
	public void TFR_MAP_002_V02_001() {
		boolean error;
		error = tav.execute("TFR-MAP-002-V02-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_002_V02_001
	
	@Test
	public void TFR_MAP_002_V04_001() {
		boolean error;
		error = tav.execute("TFR-MAP-002-V04-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_002_V04_001
	
	@Test
	public void TFR_MAP_003_V00_001() {
		boolean error;
		error = tav.execute("TFR-MAP-003-V00-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_003_V00_001
	
	@Test
	public void TFR_MAP_003_V01_001() {
		boolean error;
		error = tav.execute("TFR-MAP-003-V01-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_003_V01_001
	
	@Test
	public void TFR_MAP_003_V03_001() {
		boolean error;
		error = tav.execute("TFR-MAP-003-V03-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_003_V03_001
	
	@Test
	public void TFR_MAP_003_V04_001() {
		boolean error;
		error = tav.execute("TFR-MAP-003-V04-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_003_V04_001
	
	@Test
	public void TFR_MAP_004_V00_001() {
		boolean error;
		error = tav.execute("TFR-MAP-004-V00-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_004_V00_001
	
	@Test
	public void TFR_MAP_004_V02_001() {
		boolean error;
		error = tav.execute("TFR-MAP-004-V02-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_004_V02_001
	
	@Test
	public void TFR_MAP_005_V00_001() {
		boolean error;
		error = tav.execute("TFR-MAP-005-V00-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_005_V00_001
	
	@Test
	public void TFR_MAP_005_V01_001() {
		boolean error;
		error = tav.execute("TFR-MAP-005-V01-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_005_V01_001
	
	@Test
	public void TFR_MAP_005_V02_001() {
		boolean error;
		error = tav.execute("TFR-MAP-005-V02-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_005_V02_001
	
	@Test
	public void TFR_MAP_005_V02_002() {
		boolean error;
		error = tav.execute("TFR-MAP-005-V02-002", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_005_V02_002
	
	@Test
	public void TFR_MAP_005_V03_001() {
		boolean error;
		error = tav.execute("TFR-MAP-005-V03-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_005_V03_001
	
	@Test
	public void TFR_MAP_005_V04_001() {
		boolean error;
		error = tav.execute("TFR-MAP-005-V04-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_005_V04_001
	
	@Test
	public void TFR_MAP_005_V04_002() {
		boolean error;
		error = tav.execute("TFR-MAP-005-V04-002", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_005_V04_002
	
	@Test
	public void TFR_MAP_006_V00_001() {
		boolean error;
		error = tav.execute("TFR-MAP-006-V00-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_006_V00_001
	
	@Test
	public void TFR_MAP_007_V00_001() {
		boolean error;
		error = tav.execute("TFR-MAP-007-V00-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_007_V00_001
	
	@Test
	public void TFR_MAP_007_V01_001() {
		boolean error;
		error = tav.execute("TFR-MAP-007-V01-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_007_V01_001

	@Test
	public void TFR_MAP_007_V02_001() {
		boolean error;
		error = tav.execute("TFR-MAP-007-V02-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_007_V02_001
	
	@Test
	public void TFR_MAP_007_V02_002() {
		boolean error;
		error = tav.execute("TFR-MAP-007-V02-002", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_007_V02_002
	
	@Test
	public void TFR_MAP_007_V03_001() {
		boolean error;
		error = tav.execute("TFR-MAP-007-V03-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_007_V03_001
	
	@Test
	public void TFR_MAP_007_V04_001() {
		boolean error;
		error = tav.execute("TFR-MAP-007-V04-001", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_007_V04_001
	
	@Test
	public void TFR_MAP_007_V04_002() {
		boolean error;
		error = tav.execute("TFR-MAP-007-V04-002", ol);
		if(error == false)
			nHits++;
		else
			nFailures++;
		nTests++;
		assertFalse(error);
	}// end TFR_MAP_007_V04_002

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		OutputLog.endLog(ol);
		
		elapsed = System.currentTimeMillis() - start;
		timeTests = (double) elapsed / 1000.0;
		
		// prints a little summary of test results
		System.out.println("\nTEST SUMMARY:\n");
		System.out.println("Time elapsed: " + timeTests + " seconds");
		System.out.println("Tests executed: " + nTests);
		System.out.println("Hits: " + nHits);
		System.out.print("Failures: " + nFailures);
		System.out.println("  which " + tav.nErrors + " are errors");
		
		// if has failures, prints error log
		if(nFailures > 0) {
			FileReader errLog = new FileReader("Sketcher2UIM-ErrorLog.txt");
			BufferedReader buf = new BufferedReader(errLog);
			String line;
		
			while((line = buf.readLine()) != null) {
				System.out.println(line);
			}

			buf.close();
		}
	}
}
